$(document).ready(function() {
  $(document.body).on('click', '.link_to_modal:not(.mw-link)', function(e) {
    e.preventDefault();
    $(this).modal_window('click_to_link');
  });
  $(document).ajaxStop(function() {
    // destroy orphans windows (who has no parent link)
    $("body > div.modal_window").modal_window('destroy_if_needed');
  });
});

!function($) {
  "use strict"; // jshint ;_;

  var modal_window = function(element, options) {
    this.current_id = RMPlus.Utils.create_guid();
    this.$element = $(element);
    this.$element.addClass('mw-link');
    this.options = $.extend(true, {}, $.fn.modal_window.defaults, options || {});

    this.loaded = false;

    this.detect_link_options();
    this.detect_modal_window();
    this.detect_mask();

    if (this.options.link_opts.over_parent_window) {
      this.$parent_window = this.$element.closest('div.modal_window');
      if (this.$parent_window.length == 0) {
        this.$parent_window = undefined;
        this.options.link_opts.over_parent_window = false;
      }
    }

    this.visible = false;
    this.loading = false;
    this.listen();
  };

  modal_window.prototype = {
    constructor: modal_window,
    detect_link_options: function() {
      var classes = this.$element.attr('class');
      if (classes && classes.length > 0) {
        classes = classes.split(' ');
        var cl = '';
        for (var sch = 0; sch < classes.length; sch ++) {
          cl = classes[sch].replace('-', '_');
          if (this.options.link_opts[cl] == false) {
            this.options.link_opts[cl] = true;
          }
        }
      }
    },
    detect_modal_window: function() {
      if (this.$window) { return; }

      var id = this.$element.attr('id');
      if (!id || id.length == 0) {
        id = RMPlus.Utils.create_guid();
      }
      var $div = $("#modal-" + id);

      if ($div.length == 0) {
        this.$window = $("<div id='modal-" + id + "' class='modal_window" + (this.options.link_opts.click_out ? ' click_out' : '') + "'></div>");
      } else {
        this.$window = $div;
        if (this.$window.text() != '') { this.loaded = true; }
      }
      this.$window.css('z-index', $.fn.modal_window.z_index);
      this.$window.data('modal_window', this);
      $(document.body).prepend(this.$window);
    },
    detect_mask: function() {
      if (this.$mask) { return; }

      this.$mask = $("<div class='mw-drop-mask'></div>");
      $(document.body).prepend(this.$mask);
    },
    listen: function() {
      this.$element.on('click.modal_window', $.proxy(this.click_to_link, this));
      if (!this.options.link_opts.click_out) {
        this.$window.on('mouseleave.modal_window', $.proxy(this.mouse_leave_window, this));
      }
      this.$mask.on('click.modal_window', $.proxy(function() {
        this.hide();
      }, this));
    },

    // EVENTS //
    click_to_link: function(e) {
      if (e) { e.preventDefault(); }
      if (this.options.link_opts.over_parent_window) {
        this.hide_windows(this.$parent_window);
        this.$window.zIndex(this.$parent_window.zIndex() + 2);
      } else {
        this.hide_windows();
      }

      if (this.options.link_opts.refreshable || (!this.loaded && !this.options.link_opts.static_content_only)) {
        this.load();
      } else {
        this.show();
      }
    },
    mouse_leave_window: function(evt) {
      if (evt.target.nodeName.toLowerCase() != 'select' && !$.contains(this.$window[0], evt.target)) {
        this.hide();
      }
    },
    // END EVENTS //

    hide_windows: function($parent) {
      $parent = $parent || $(document.body);
      $parent.find('.modal_window:visible').modal_window('hide');
    },

    show: function() {
      var scroll = { top: $(document).scrollTop(), left: $(document).scrollLeft() };
      var offset = this.$element.offset();
      offset.top = offset.top - scroll.top;
      offset.left = offset.left - scroll.left;
      offset.width = this.$element.outerWidth();
      offset.height = this.$element.outerHeight();

      var mw_width = this.$window.outerWidth();
      var mw_height = this.$window.outerHeight();
      var doc_w = $(window).outerWidth();
      var doc_h = $(window).outerHeight();
      var margin = 7;

      var mw_class_lr = this.options.window_right_class;
      var mw_class_tb = this.options.window_bottom_class;
      var mw_block = '';

      if (this.options.link_opts.block_preferred) {
        mw_block = this.options.window_block_class;
        if ((this.options.link_opts.left_preferred || doc_w < mw_width + offset.left) && mw_width < offset.left + offset.width) {
          offset.left = offset.left + offset.width + margin;
        } else { offset.width = -margin; }

        if ((this.options.link_opts.top_preferred || doc_h < mw_height + offset.top + offset.height) && mw_height < offset.top) {
          offset.height = 0;
        } else { offset.top = offset.top + offset.height; }
      }

      // right from element - is default
      this.$window.css('left', offset.left + offset.width + margin + scroll.left);

      if (this.options.link_opts.left_preferred || doc_w < mw_width + offset.left + offset.width + margin) {
        // try to display left if preferred left or no space at right
        if (mw_width < offset.left) {
          this.$window.css('left', offset.left - margin - mw_width + scroll.left);
          mw_class_lr = this.options.window_left_class;
        }
      }


      // vertical position - default down
      this.$window.css('top', offset.top + scroll.top);

      if (this.options.link_opts.top_preferred || doc_h < offset.top + mw_height - offset.height) {
        // try to display as preferred no space bottom or top-preferred and
        if (mw_height < offset.top + offset.height) {
          this.$window.css('top', offset.top + scroll.top + offset.height - mw_height);
          mw_class_tb = this.options.window_top_class;
        }
      }

      this.$window.removeClass(this.options.window_top_class + ' ' + this.options.window_bottom_class + ' ' + this.options.window_left_class + ' ' + this.options.window_right_class + ' ' + this.options.window_block_class);
      this.$window.addClass(mw_class_lr).addClass(mw_class_tb).addClass(mw_block);

      this.$window.show().trigger('modal_window_shown');
      this.show_mask();
      this.$element.trigger('modal_window_shown');
      this.visible = true;

      this.$window.find('input, select, textarea').filter(':visible:first').focus();
    },
    hide: function() {
      if (this.visible) {
        this.$window.hide().trigger('modal_window_hidden');
        this.$element.trigger('modal_window_hidden');
        this.hide_mask();
        this.visible = false;
      }
    },
    show_mask: function() {
      if (this.$window.prev()[0] !== this.$mask[0]) {
        this.$window.before(this.$mask);
      }
      this.$mask.zIndex(this.$window.zIndex() - 1);
      this.$mask.css(this._make_mask_css()).addClass('mw-mask-show');

      $(window).on('resize.modal_window.' + this.current_id, $.proxy(function() {
        this.$mask.css(this._make_mask_css());
      }, this));
    },
    hide_mask: function() {
      this.$mask.removeClass('mw-mask-show');
      $(window).off('resize.modal_window.' + this.current_id);
    },
    load: function() {
      if (this.loading) { return; }
      this.loading = true;
      this.append_loader();

      if (this.$element[0].tagName == 'A') {
        $.ajax({method: (this.$element.attr('data-method') || this.$element.attr('data-modal-method') || '').toUpperCase() || 'GET', url: this.$element.attr('href')}).always($.proxy(function(html) { this.complete_load(html.responseText || html); }, this));
      } else {
        var $frm = this.$element.closest('form');
        $.ajax({ method: ($frm.attr('method') || '').toUpperCase() || 'GET', url: $frm.attr('action'), data: $frm.serialize() }).always($.proxy(function(html) { this.complete_load(html.responseText || html); }, this));
      }
    },
    append_loader: function() {
      var hide_obj = this.$element;
      if (this.options.link_opts.parent_loader) {
        hide_obj = this.$element.closest('.parent_loader');
      }
      var offset = hide_obj.offset();
      var h = hide_obj.outerHeight();
      var w = hide_obj.outerWidth();
      hide_obj.addClass('invisible_link');
      this.$loader_object = hide_obj;

      if (!this.$loader) {
        this.$loader = $('<div class="mw_loader loader">&nbsp;</div>');
      }
      $(document.body).append(this.$loader);
      this.$loader.css('left', offset.left)
          .css('top', offset.top)
          .css('width', w)
          .css('height', h)
          .show();
    },
    remove_loader: function() {
      if (this.$loader_object) {
        this.$loader_object.removeClass('invisible_link');
        this.$loader_object = undefined;
      }

      if (this.$loader) {
        this.$loader.remove();
      }
    },
    complete_load: function(result_data) {
      this.remove_loader();
      this.$window.html(result_data);
      this.loading = false;
      this.loaded = true;
      this.show();
    },
    destroy_if_needed: function() {
      if (this.$window && this.$element && !$.contains(document.documentElement, this.$element[0])) {
        this.destroy();
      }
    },
    destroy: function() {
      this.$window.remove();
      this.$window = undefined;
      this.$mask.remove();
      this.$mask = undefined;
      this.$element.removeClass('mw-link');
      this.$element.off('.modal_window').removeData('modal_window');
      $(window).off('resize.modal_window.' + this.current_id);
      this.$element = undefined;
    },


    _make_mask_css: function() {
      return {
        width  : Math.max(document.documentElement.scrollWidth,  $(window).width()),
        height : Math.max(document.documentElement.scrollHeight, $(window).height())
      }
    }
  };

  $.fn.modal_window = function(option) {
    return this.each(function() {
      var $this = $(this)
          , data = $this.data('modal_window')
          , options = typeof option == 'object' && option;
      if (!data) {
        if ($this.hasClass('modal_window')) { return; }
        $this.data('modal_window', (data = new modal_window(this, options)));
      }
      if (typeof option == 'string') { data[option]( ); }
    });
  };

  $.fn.modal_window.defaults = {
    window_top_class: 'mw-top',
    window_bottom_class: 'mw-bottom',
    window_left_class: 'mw-left',
    window_right_class: 'mw-right',
    window_block_class: 'mw-block',
    link_opts: {
      click_out: false,
      over_parent_window: false,
      static_content_only: false,
      refreshable: false,
      parent_loader: false,
      block_preferred: false,
      left_preferred: false,
      top_preferred: false
    }
  };
  $.fn.modal_window.z_index = 1100;

  $.fn.modal_window.Constructor = modal_window;
} (window.jQuery);


